==> zen-0.0.1.tar.gz: zen-0.0.1/src/parser/states.c <==
/** 
 * Functions to handle the text styles and part alignments that can appear 
 * in the html page. These takes care of stacking old states, so that they 
 * can easily be retreived again.
 *
 * @author Tomas Berndtsson <tomas@nocrew.org>
 */

/*
 * Copyright (C) 1999, Tomas Berndtsson <tomas@nocrew.org>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <stdlib.h>
#include <stdio.h>

#include "ui.h"
#include "layout.h"
#include "states.h"
#include "helpers.h"

/* A pointer to the current style state. */
static struct parse_state_style *current_style = NULL;

/* A pointer to the current style state. */
static struct parse_state_align *current_align = NULL;

/* A pointer to the current style state. */
static struct parse_state_base *current_base = NULL;

/**
 * Initialize the state stack with the default values.
 *
 * @return non-zero value if an error occurred.
 */
int parse_state_init(void)
{
  if(current_style == NULL) {
    current_style = 
      (struct parse_state_style *)malloc(sizeof(struct parse_state_style));
    if(current_style == NULL)
      return 1;
  }
    
  if(current_align == NULL) {
    current_align = 
      (struct parse_state_align *)malloc(sizeof(struct parse_state_align));
    if(current_align == NULL) {
      free(current_style);
      return 1;
    }
  }

  if(current_base == NULL) {
    current_base = 
      (struct parse_state_base *)malloc(sizeof(struct parse_state_base));
    if(current_base == NULL) {
      free(current_style);
      free(current_align);
      return 1;
    }
  }

  current_style->style.size = user_interface.ui_settings.default_fontsize;
  current_style->style.colour = 0x80ffffff;
  current_style->style.monospaced = 0;
  current_style->style.italic = 0;
  current_style->style.bold = 0;
  current_style->style.underlined = 0;
  current_style->style.overlined = 0;
  current_style->style.subscript = 0;
  current_style->style.superscript = 0;
  current_style->style.preformatted = 0;
  current_style->previous = NULL;

  current_align->align.horizontal = LAYOUT_PART_ALIGN_LEFT;
  current_align->align.vertical = LAYOUT_PART_ALIGN_BOTTOM;
  current_align->previous = NULL;

  current_base->base = NULL;
  current_base->previous = NULL;

  return 0;
}

/**
 * Copy the state values from the current state to the given structs.
 *
 * @param style A pointer to where to store text style information. A NULL
 * @param style value means the style will not be retreived.
 * @param align A pointer to where to store part align information. A NULL
 * @param align value means the alignment will not be retreived.
 * @param base A pointer to a layout part which is the base of parts
 * @param base used by the current set of tags. This copies the 
 * @param base pointer, as opposed to the style and align which copy
 * @param base the actual structs.
 *
 * @return non-zero value if an error occurred.
 */
int parse_state_get_current(struct layout_text_styles *style,
			    struct layout_aligns *align,
			    struct layout_part **base)
{
  if(style)
    *style = current_style->style;
  
  if(align)
    *align = current_align->align;

  if(base)
    *base = current_base->base;

  return 0;
}

/**
 * Copy the style and align values into a new state and place that first 
 * on the stack. Also sets the new mother of layout parts, if given.
 *
 * @param style A pointer to the style to store in the state. A NULL value
 * @param style means the style will be the same as the previous state.
 * @param align A pointer to the align to store in the state. A NULL value
 * @param align means the align will be the same as the previous state.
 * @param base A pointer to a layout part which is to be the base part
 * @param base of new parts added to the linked list.
 *
 * @return non-zero value if an error occurred.
 */
int parse_state_push(struct layout_text_styles *style,
		     struct layout_aligns *align,
		     struct layout_part **base)
{
  struct parse_state_style *new_style = NULL;
  struct parse_state_align *new_align = NULL;
  struct parse_state_base *new_base = NULL;

  /* If we happen to pop more than we push, we will come to a position
   * where current_state is NULL again. This cannot be allowed, and to
   * easily recover from this problem, we simply reinitialize it.
   */
  if(current_style == NULL || current_align == NULL || current_base == NULL) {
    fprintf(stderr, "Warning! Initialized the state more than once!\n");
    parse_state_init();
  }

  if(style) {
    new_style = 
      (struct parse_state_style *)malloc(sizeof(struct parse_state_style));
    if(new_style == NULL)
      return 1;
    new_style->style = *style;
    new_style->previous = current_style;
    current_style = new_style;
  }

  if(align) {
    new_align = 
      (struct parse_state_align *)malloc(sizeof(struct parse_state_align));
    if(new_align == NULL) {
      if(new_style != NULL)
	free(new_style);
      return 1;
    }
    new_align->align = *align;
    new_align->previous = current_align;
    current_align = new_align;
  }

  if(base) {
    new_base = 
      (struct parse_state_base *)malloc(sizeof(struct parse_state_base));
    if(new_base == NULL) {
      if(new_style != NULL)
	free(new_style);
      if(new_align != NULL)
	free(new_align);
      return 1;
    }
    new_base->base = *base;
    new_base->previous = current_base;
    current_base = new_base;
  }

  /* We need special treatment of the preformatted style, because this
   * inflicts on the parsing.
   */
  parse_string_set_preformatted(current_style->style.preformatted);

  return 0;
}

/**
 * Remove the first state from the stack, no matter what there is inside it.
 *
 * @member pop_style Non-zero value if popping the current style.
 * @member pop_align Non-zero value if popping the current align.
 * @member pop_base Non-zero value if popping the current base.
 *
 * @return non-zero value if an error occurred.
 */
int parse_state_pop(int pop_style, int pop_align, int pop_base)
{
  if(pop_style && current_style != NULL) {
    struct parse_state_style *old_style = current_style;
    current_style = current_style->previous;
    free(old_style);
  }

  if(pop_align && current_align != NULL) {
    struct parse_state_align *old_align = current_align;
    current_align = current_align->previous;
    free(old_align);
  }

  if(pop_base && current_base != NULL) {
    struct parse_state_base *old_base = current_base;
    current_base = current_base->previous;
    free(old_base);
  }

  /* We need special treatment of the preformatted style, because this
   * inflicts on the parsing.
   */
  if(current_style)
    parse_string_set_preformatted(current_style->style.preformatted);

  return 0;
}

==> zen-0.0.2.tar.gz: zen-0.0.2/src/parser/states.c <==
/** 
 * Functions to handle the text styles and part alignments that can appear 
 * in the html page. These takes care of stacking old states, so that they 
 * can easily be retreived again.
 *
 * @author Tomas Berndtsson <tomas@nocrew.org>
 */

/*
 * Copyright (C) 1999, Tomas Berndtsson <tomas@nocrew.org>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <stdlib.h>
#include <stdio.h>

#include "ui.h"
#include "layout.h"
#include "states.h"
#include "helpers.h"

/* A pointer to the current style state. */
static struct parse_state_style *current_style = NULL;

/* A pointer to the current style state. */
static struct parse_state_align *current_align = NULL;

/* A pointer to the current style state. */
static struct parse_state_base *current_base = NULL;

/* A pointer to the current state mark. */
static struct parse_state_mark *current_mark = NULL;

/**
 * Reset the different states to their default values. This is
 * used when initializing the states, and can also be used later
 * to bring the state to its default state. 
 *
 * @param style A pointer to a text style struct to be reset.
 * @param style NULL means the text style is not be reset.
 * @param align A pointer to an alignment struct to be reset.
 * @param align NULL means the alignment is not be reset.
 * @param base A pointer to a pointer of a base part to be reset.
 * @param base NULL means the base part is not be reset.
 */
void parse_state_reset(struct layout_text_styles *style,
		       struct layout_aligns *align,
		       struct layout_part **base)
{
  if(style != NULL) {
    style->size = user_interface.ui_settings.default_fontsize;
    /*     style->face = NULL; */
    style->colour = 0x80ffffff;
    style->monospaced = 0;
    style->italic = 0;
    style->bold = 0;
    style->underlined = 0;
    style->overlined = 0;
    style->subscript = 0;
    style->superscript = 0;
    style->preformatted = 0;
  }

  if(align != NULL) {
    align->horizontal = LAYOUT_PART_ALIGN_LEFT;
    align->vertical = LAYOUT_PART_ALIGN_BOTTOM;
  }

  if(base != NULL) {
    *base = NULL;
  }
}

/**
 * Initialize the state stack with the default values.
 *
 * @return non-zero value if an error occurred.
 */
int parse_state_init(void)
{
  if(current_style == NULL) {
    current_style = 
      (struct parse_state_style *)malloc(sizeof(struct parse_state_style));
    if(current_style == NULL)
      return 1;

    parse_state_reset(&current_style->style, NULL, NULL);
    current_style->previous = NULL;
  }
    
  if(current_align == NULL) {
    current_align = 
      (struct parse_state_align *)malloc(sizeof(struct parse_state_align));
    if(current_align == NULL) {
      free(current_style);
      return 1;
    }

    parse_state_reset(NULL, &current_align->align, NULL);
    current_align->previous = NULL;
  }

  if(current_base == NULL) {
    current_base = 
      (struct parse_state_base *)malloc(sizeof(struct parse_state_base));
    if(current_base == NULL) {
      free(current_style);
      free(current_align);
      return 1;
    }

    parse_state_reset(NULL, NULL, &current_base->base);
    current_base->previous = NULL;
  }

  return 0;
}

/**
 * Delete all previous states.
 *
 * @return non-zero value if an error occurred.
 */
int parse_state_delete_all(void)
{
  while(current_style || current_align || current_base) {
    if(current_style != NULL) {
      struct parse_state_style *old_style = current_style;
      current_style = current_style->previous;
      free(old_style);
    }
    
    if(current_align != NULL) {
      struct parse_state_align *old_align = current_align;
      current_align = current_align->previous;
      free(old_align);
    }
    
    if(current_base != NULL) {
      struct parse_state_base *old_base = current_base;
      current_base = current_base->previous;
      free(old_base);
    }  
  }

  return 0;
}

/**
 * Copy the state values from the current state to the given structs.
 *
 * @param style A pointer to where to store text style information. A NULL
 * @param style value means the style will not be retreived.
 * @param align A pointer to where to store part align information. A NULL
 * @param align value means the alignment will not be retreived.
 * @param base A pointer to a layout part which is the base of parts
 * @param base used by the current set of tags. This copies the 
 * @param base pointer, as opposed to the style and align which copy
 * @param base the actual structs.
 *
 * @return non-zero value if an error occurred.
 */
int parse_state_get_current(struct layout_text_styles *style,
			    struct layout_aligns *align,
			    struct layout_part **base)
{
  if(style)
    *style = current_style->style;
  
  if(align)
    *align = current_align->align;

  if(base)
    *base = current_base->base;

  return 0;
}

/**
 * Copy the style and align values into a new state and place that first 
 * on the stack. Also sets the new mother of layout parts, if given.
 *
 * @param style A pointer to the style to store in the state. A NULL value
 * @param style means the style will be the same as the previous state.
 * @param align A pointer to the align to store in the state. A NULL value
 * @param align means the align will be the same as the previous state.
 * @param base A pointer to a layout part which is to be the base part
 * @param base of new parts added to the linked list.
 *
 * @return non-zero value if an error occurred.
 */
int parse_state_push(struct layout_text_styles *style,
		     struct layout_aligns *align,
		     struct layout_part **base)
{
  struct parse_state_style *new_style = NULL;
  struct parse_state_align *new_align = NULL;
  struct parse_state_base *new_base = NULL;

  if(style) {
    new_style = 
      (struct parse_state_style *)malloc(sizeof(struct parse_state_style));
    if(new_style == NULL)
      return 1;
    new_style->style = *style;
    new_style->previous = current_style;
    current_style = new_style;
  }

  if(align) {
    new_align = 
      (struct parse_state_align *)malloc(sizeof(struct parse_state_align));
    if(new_align == NULL) {
      if(new_style != NULL)
	free(new_style);
      return 1;
    }
    new_align->align = *align;
    new_align->previous = current_align;
    current_align = new_align;
  }

  if(base) {
    new_base = 
      (struct parse_state_base *)malloc(sizeof(struct parse_state_base));
    if(new_base == NULL) {
      if(new_style != NULL)
	free(new_style);
      if(new_align != NULL)
	free(new_align);
      return 1;
    }
    new_base->base = *base;
    new_base->previous = current_base;
    current_base = new_base;
  }

  /* We need special treatment of the preformatted style, because this
   * inflicts on the parsing.
   */
  parse_string_set_preformatted(current_style->style.preformatted);

  return 0;
}

/**
 * Remove the first state from the stack, no matter what there is inside it.
 * Only do this if we do not pop over a set state mark.
 *
 * @member pop_style Non-zero value if popping the current style.
 * @member pop_align Non-zero value if popping the current align.
 * @member pop_base Non-zero value if popping the current base.
 *
 * @return non-zero value if an error occurred.
 */
int parse_state_pop(int pop_style, int pop_align, int pop_base)
{
  if(pop_style && current_style != NULL) {
    struct parse_state_style *old_style = current_style;

    /* Do not pop over a mark limit. */
    if(!(current_mark && current_style == current_mark->mark_style)) {
      current_style = current_style->previous;
      free(old_style);
    }
  }

  if(pop_align && current_align != NULL) {
    struct parse_state_align *old_align = current_align;

    /* Do not pop over a mark limit. */
    if(!(current_mark && current_align == current_mark->mark_align)) {
      current_align = current_align->previous;
      free(old_align);
    }
  }

  if(pop_base && current_base != NULL) {
    struct parse_state_base *old_base = current_base;

    /* Do not pop over a mark limit. */
    if(!(current_mark && current_base == current_mark->mark_base)) {
      current_base = current_base->previous;
      free(old_base);
    }
  }

  /* If we happen to pop more than we push, we will come to a position
   * where current_state is NULL again. This cannot be allowed, and to
   * easily recover from this problem, we simply reinitialize it.
   */
  if(current_style == NULL || current_align == NULL || current_base == NULL) {
    fprintf(stderr, "Warning! Initialized the state more than once!\n");
    parse_state_init();
  }

  /* We need special treatment of the preformatted style, because this
   * inflicts on the parsing.
   */
  if(current_style)
    parse_string_set_preformatted(current_style->style.preformatted);

  return 0;
}

/**
 * Set a mark in the state stack. This will save the current state pointers
 * on a different stack, and the state popping function makes sure nothing
 * is popped over the current mark. This is used for tags which need their
 * own state stack, such as tables.
 *
 * @return non-zero value if an error occurred.
 */
int parse_state_set_mark(void)
{
  struct parse_state_mark *new_mark = NULL; 

  new_mark = 
    (struct parse_state_mark *)malloc(sizeof(struct parse_state_mark));
  if(new_mark == NULL)
    return 1;
  
  new_mark->mark_style = current_style;
  new_mark->mark_align = current_align;
  new_mark->mark_base = current_base;

  new_mark->previous = current_mark;
  current_mark = new_mark;

  return 0;
}

/**
 * Pop all states up until the current mark is reached.
 *
 * @return non-zero value if an error occurred.
 */
int parse_state_unset_mark(void)
{
  struct parse_state_mark *old_mark = current_mark;
  int all_flushed, pop_style, pop_align, pop_base;

  if(current_mark == NULL) {
    fprintf(stderr, "Warning! Trying to unset more marks than there are!\n");
    return 1;
  }

  all_flushed = 0;
  while(!all_flushed) {
    if(current_style != current_mark->mark_style)
      pop_style = 1;
    else
      pop_style = 0;

    if(current_align != current_mark->mark_align)
      pop_align = 1;
    else
      pop_align = 0;

    if(current_base != current_mark->mark_base)
      pop_base = 1;
    else
      pop_base = 0;

    if(pop_style == 0 && pop_align == 0 && pop_base == 0)
      all_flushed = 1;
    else
      parse_state_pop(pop_style, pop_align, pop_base);
  }

  current_mark = current_mark->previous;
  free(old_mark);
  
  return 0;
}
